/*------------------------------------------------------------------------
Module:        mime_alter.c
Author:        Paul L Daniels / Steven Coutts
Project:       ripMIME
State:         development
Creation Date: 01/05/2001
Description:   MIME-alter is an additional module in the mime.[ch] object, which will contain various functions which will -alter- mime-packs.
------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <syslog.h>
#include <errno.h>

#include "strlower.h"


/* Our content types */
#define _unknown -1
#define _none 0
#define _text_plain 1
#define _text_html 2
#define _text_print 3
#define _text_unknown 4

/* Our encoding scemes */
#define _base64 1
#define _print 2
#define _7bit 3

#define _BUFFER_MAX 1024

/* Globals */
char *buffer[_BUFFER_MAX]; /* allocate a static array of 1024 pointers to char */
unsigned char encode64[64]; /* binary -> base64 encoding table */

int __verbose=0;				/* do we talk as we walk */
int __paranoid=1;				/* set paranoid to yes! */
int __HTMLtoo=1;

int AM_set_paranoid( int level )
{
	__paranoid = level;
	return __paranoid;
}


int AM_set_verbose( int level )
{
	__verbose = level;
	return __verbose;
}

int AM_set_HTMLtoo( int level )
{
	__HTMLtoo = level;
	return __HTMLtoo;
}

/*------------------------------------------------------------------------
Procedure:     strlower ID:1
Purpose:       Converts a given string into all lowercase
Input:         convertme : String to be converted
Output:        none
Errors:
------------------------------------------------------------------------*/
int AM_strlower( char *convertme ){

	static char c;

	c = *convertme;

	while ( c != '\0') {
		if (( c >= 'A' ) && ( c <= 'Z')) (*convertme) += 32; /* if C is uppercase, convert down */
		c = *(++convertme); /* pre-increment, and then get the value of C */
	}

	return 0;
}





/*------------------------------------------------------------------------
Procedure:     AM_create_encode_table ID:1
Purpose:       Builds the value:ASCII relationship table for BASE64 -encoding-
process
Input:         none
Output:
Errors:
------------------------------------------------------------------------*/
int AM_create_encode_table( void )
{

	int i,j;

	for ( i = 'A', j=0; i <= 'Z'; i++, j++)
	{
		encode64[j]=i;
	}

	for ( i = 'a'; i <= 'z'; i++, j++)
	{
		encode64[j]=i;
	}

	for ( i = '0'; i <= '9'; i++, j++)
	{
		encode64[j]=i;
	}

	encode64[62] = '+';
	encode64[63] = '/';
	return 0;
}







/*------------------------------------------------------------------------
Procedure:     AM_base64_encodef ID:1
Purpose:       encode to BASE64 an input stream to an output stream
Input:         FILE *fin: Input stream
FILE *fout: Output stream
Output:
Errors:
------------------------------------------------------------------------*/
int AM_base64_encodef( FILE *fin, FILE *fout )
{
	unsigned char inbuf[3];
	unsigned char outbuf[4];
	int cc;
	int byte_count;

	if (!fin)
	{
		syslog(1,"AM_base64_encodef: Error: input file stream not open, please use AM_base64_encode(infile,outfile)");
		return -1;
	}

	if (!fout)
	{
		syslog(1,"AM_base64_encodef: Error: output file stream not open, please use AM_base64_encode(infile,outfile)");
		return -1;
	}

	AM_create_encode_table(); /* setup the binary->base64 encoding table */

	cc = 0;	/* Character Count = 0 */

	while ((byte_count = fread(inbuf, 1, 3, fin)))	/* while we get more than 0 bytes */
	{
		/* Split 3 bytes into 4 bytes by taking 6bit blocks out of the 24 bit (3x8bit)
		* array */
		outbuf[0] = encode64[((inbuf[0] & 0xFC) >> 2)];
		outbuf[1] = encode64[((inbuf[0] & 0x03) << 4) | ((inbuf[1] & 0xF0) >> 4)];
		outbuf[2] = encode64[((inbuf[1] & 0x0F) << 2) | ((inbuf[2] & 0xC0) >> 6)];
		outbuf[3] = encode64[(inbuf[2] & 0x3F)];

		/* if we didn't get a full 3 bytes from the file read, then we need to then
		* pad the output with '=' (base64 padding char) */
		if ( byte_count < 3 )
		{

			if ( byte_count == 2 ) outbuf[3] = '=';
			if ( byte_count == 1 ) outbuf[3] = outbuf[2] = '=';

			fwrite(outbuf, 1, 4, fout); 	/* write out the buffer */
			fwrite( "\n", 1, 1, fout ); 	/* as this is the last line, put a trailing \n */
			break; 								/* exit out of the while loop */
		}

		fwrite(outbuf, 1, 4, fout);		/* normal operation buffer-write */

		cc += 4;									/* increment the output char count by 4 */

		if (cc > 76)							/* if we have more than 76 chars, then put in a \n */
		{
			fwrite( "\n", 1, 1, fout );	/* write the \n out */
			cc = 0;								/* reset the output char count */
		} /* if bytecount == 3 */
	} /* while more data to read in */

	return 0;
}






/*------------------------------------------------------------------------
Procedure:     AM_base64_encode ID:1
Purpose:       Encodes a given input stream into BASE64 and outputs to a given
output stream
Input:         char *enc_fname: Input filename of the stream to encode
char *out_fname: Output filename of the stream to send to
Output:
Errors:
------------------------------------------------------------------------*/
int AM_base64_encode( char *enc_fname, char *out_fname )
{
	FILE *fin, *fout;

	fin = fopen( enc_fname, "rb" );
	if (!fin)
	{
		syslog(1,"AM_base64_encode: Cannot open \"%s\" for reading.(%s)",enc_fname,strerror(errno));
		return -1;
	}

	fout = fopen ( out_fname, "wb" );
	if (!fout)
	{
		syslog(1,"AM_base64_encode: Cannot open \"%s\" for writing.(%s)",out_fname,strerror(errno));
		fclose(fin);
		return -1;
	}

	AM_base64_encodef( fin, fout );	/* encode and output */

	fclose(fin);	/* close the input file */
	fclose(fout);	/* close the output file */

	return 0;
}




/*------------------------------------------------------------------------
Procedure:     AM_quick_clean_filename ID:1
Purpose:       scans all the chars of a filename, converts any non-safe chars to
a '_'
Input:         char *filename: The filename we're going to convert/scan
Output:
Errors:
------------------------------------------------------------------------*/
void AM_quick_clean_filename( char *fname )
{
	char tmp[1024];
	char *p = strrchr(fname,'/');

	/* scan out any directory separators */
	if (p)
	{
		p++;
		sprintf(tmp,"%s",p);
		strcpy(fname,p);
	}
	else if ( (p = strrchr(fname,'\\')))
	{
		p++;
		sprintf(tmp,"%s",p);
		strcpy(fname,p);
	}


	while (*fname) {
		if ( !isalnum(*fname)||(*fname == ' ') ) *fname='_';
		fname++;
	}
}






/*------------------------------------------------------------------------
Procedure:     AM_attachment_replace ID:1
Purpose:       Replace the attachment in a mailbody
Input:
Output:
Errors:
------------------------------------------------------------------------*/
int AM_attachment_replace( char *mpack_name, char *old_att_name, char *new_att_name, char *att_path )
{
	FILE *in, *tmp, *newa;
	char *ofn, *nfn;
	char line[1024];
	char lline[1024];
	char lname[1024];
	char ts[1024];
	char tname[1024];
	char *p, *q;
	int base64hit = 0;
	int hit = 0;
	int quote_on = 0;
	long header_start = 0;

	in = fopen( mpack_name, "r" ); /* open up our original mailpack */

	if (!in)	/* handle file opening error */
	{
		syslog(1,"AM_attachment_replace: Cannot open mailpack %s for reading, \"%s\""\
		,mpack_name\
		,strerror(errno));
		return -1;
	}

	ofn = strrchr(old_att_name,'/');  	/* get the attachment name out of the full path */
	if (!ofn) ofn = old_att_name;			/* if there was no '/', then reset the ofn ptr */
	else ofn++; 								/* skip the last '/' */
	strcpy(lname, ofn); 						/* create a lowercase version */
	strlower(lname);
	AM_quick_clean_filename(lname); 		/* clean up the filename */

	nfn = strrchr(new_att_name,'/');		/* extract new att name from path */
	if (!nfn) nfn = new_att_name;			/* if there was no '/' then reset the nfn ptr */
	else nfn++;

	sprintf(tname,"%s.tmp", mpack_name);/* create the temporary mailpack name */
	tmp = fopen( tname, "w" );				/* try open the temp file */

	if (!tmp) /* handle file opening error */
	{
		syslog(1,"AM_attachment_replace: Cannot open temporary file %s for writing, \"%s\""\
		,tname\
		,strerror(errno));
		return -1;
	}


	while (!feof(in))
	{
		hit = 0;
		base64hit = 0;
		quote_on = 0;
		header_start = 0;

		/* Step 1. first locate our line that has the filename that we want */
		/* Step 1. first locate our line that has the filename that we want */
		/* Step 1. first locate our line that has the filename that we want */
		while (fgets(line, 1024, in))
		{
			/* if we haven't yet set the start of the headers, then we better */
			if (header_start == 0) header_start = ftell(in);

			strcpy(lline,line); /* copy and convert to lowercase the line we've read */
			strlower(lline);

			if (strstr(lline,"base64")) /* if we find BASE64, then set the flag */
				base64hit++;

			if ((p=strstr(lline,"name="))) /* if the line contains name=, then check it */
			{
				quote_on = 0;					/* no quotes found yet */
				p = p +strlen("name=");		/* move to just after the ='s */
				if (*p == '\"')				/* if there's a quote... */
				{
					quote_on++;					/* set quote to ON */
					p++;							/* move to next char */
				}

				q = strtok(p,";\"\n\r");	/* extract the filename */
				AM_quick_clean_filename(q);/* clean the filename */

				fprintf(stderr,"P1: Comparing -%s- to -%s-\n",q,lname);

				if (\
			  (base64hit)\
			 && (strlen(q) == strlen(lname))\
			&& (strncmp(q,lname, strlen(lname))==0)\
					    )
				{
					p = q;						/* set p to point to the filename */
					q = line +(p -lline);	/* move along to the same place in the normal case
					*  line */
					*q = '\0';					/* crop the line just before the filename */
					fprintf(stderr,"P1:line = %s\n",line);
					sprintf(ts, "%s%s", line, nfn); /* add in the filename */
					if (quote_on) strcat(ts,"\""); /* if there was a quote, add it too */
					strcat(ts,"\n");			/* dont forget our trailing \n */
					fprintf(stderr,"P1:Output = %s\n",ts);
					strcpy(line,ts);			/* copy the string to "line" */
					hit++;						/* flag that we had success */
				}
				else
				{
					base64hit = 0;				/* if the name didn't match, ensure we reset
					* the base64 flag, lest we accidently try to
					* encode the next attachment and it's not a
					* Base64 encoded one */
				}
			}

			fprintf(tmp,"%s",line);			/* output the line to the new temp mailpack */

			if (hit && base64hit) break; 	/* if we had success and base64, then exit */

		}

		/* If we had success with finding the right filename etc... */
		if ((hit)&&(base64hit))
		{

			/* Step 2. Locate the start of the attachment */
			/* Step 2. Locate the start of the attachment */
			/* Step 2. Locate the start of the attachment */
			while (fgets(line, 1024, in))
			{

				strcpy(lline,line);			/* copy and convert the next line */
				strlower(lline);

				if ((p=strstr(lline,"name=")))	/* if the line has name= in it... */
				{
					quote_on = 0;				/* set no quote */
					p = p +strlen("name="); /* get the position where the name starts */
					if (*p == '\"')			/* if quoted... */
					{
						quote_on++;				/* set quote flag */
						p++;						/* step one char to the right */
					}

					q = strtok(p,";\"\n\r");	/* extract the filename */
					AM_quick_clean_filename(q);/* clean the filename */

					if ( (strlen(q) == strlen(lname))\
				&& (strncmp(q,lname, strlen(lname))==0)
						    )
					{
						p = q;
						q = line +(p -lline);
						*q = '\0';
						sprintf(ts, "%s%s", line, nfn);
						if (quote_on) strcat(ts,"\"");
						strcat(ts,"\n");
						fprintf(stderr,"P2: line = %s\n",line);
						strcpy(line,ts);
					}
				}


				fprintf(tmp,"%s",line);

				if (strlen(line) < 3) break;	/* if we've gotten to a blank line, exit */
			}


			/* Step 3. Insert attachment */
			/* Step 3. Insert attachment */
			/* Step 3. Insert attachment */

			newa = fopen( att_path, "r" );		/* Open the new attachment */

			if (!newa)										/* if we couldn't open it */
			{
				syslog(1,"AM_attachment_replace: Cannot open file \"%s\" required to be inserted.(%s)"\
			,new_att_name, strerror(errno));

				/* as we couldn't open the new attachment, then we better keep the
				* existing file so that we dont bugger up the mailpack, better to have
				* the old attachment than non at all! */
				while (fgets(line, 1024, in))
				{
					fprintf(tmp,"%s",line);
					if (strpbrk(line,"-")) break;
				}

			}
			else
			    {
				AM_base64_encodef( newa, tmp );	/* Encode and output the new attachment */
				fclose(newa);								/* close the new attachment */

				/* absorb the old attachment */
				while (fgets(line, 1024, in))
				{
					if (strpbrk(line,"-"))				/* we're assuming that in base64, only
					* boundary lines have a '-' in them
					* This is possibly a bad thing (IMHO)
					* but short of tracking boundaries, I
						* have no other way */
					{
						fprintf(tmp,"%s",line);
						break;
					}
				}
			}
		}
	}

	clearerr(in);										/* clear out any errno's */
	fclose(tmp);										/* close our new mailpack */
	fclose(in);											/* close our old mailpack */
	rename( tname, mpack_name );					/* rename! */

	return 0;
}





/*------------------------------------------------------------------------
Procedure:     AM_insert_disclaimer ID:1
Purpose:       Inserts a disclaimer/footer file into a given email content stream,
on the basis that the content type/encoding are indicating that the
point in the email is a suitable (ie, non-binary-encoded) place.
Input:         inf - Input file stream, the disclaimer, as opened previously
outf - Output file stream, the new mailpack which is being created
content_type - indicates what content type was last detected
content_encoding - indicates what content encoding scheme was last detected
Output:        none
Errors:
------------------------------------------------------------------------*/
int AM_insert_disclaimer( FILE *inf, FILE *outf, int content_type, int content_encoding )
{
	char line[1024]="";

	if ((content_encoding == _none) || (content_encoding == _print) || (content_encoding == _7bit))
	{
		if (content_type == _text_html) fprintf(outf,"\n<PRE>\n");
		while (fgets(line,1024,inf)) fprintf(outf,"%s",line);
		fprintf(outf,"\n");
		if (content_type == _text_html) fprintf(outf,"</pre>\n");
	}

	return 0;
}




/*------------------------------------------------------------------------
Procedure:     AM_add_disclaimer ID:1
Purpose:       General purpose routine for adding what is typically a
disclaimer to email mailpacks.  This only inserts into the first
attachment body, or at the end of a non-encoded email
Input:         mpackname - The name of the mailpack which we are inserting into
disclaimer - The name of the file which contains the disclaimer
Output:        none
Errors:
------------------------------------------------------------------------*/
int AM_add_disclaimer( char *mpackname, char *disclaimer )
{
	FILE *mp, *d, *newf;
	char line[1024]="";
	char safe[1024]="";
	char lline[1024]="";
	char mpacktmp[1024]="";
	char mpackold[1024]="";
	char boundary[1024]="";

	int content_type = _none;
	int content_encoding = _none;

	int ishtml = 0;
	int isfile = 0;

	int result = 0;
	int boundary_found = 0;
	int disclaimer_inserted = 0;
	char *pos, *newpos;

	/* create our temp filename */
	sprintf(mpacktmp,"%s.tmp",mpackname);
	sprintf(mpackold,"%s.old",mpackname);

	/* Initialise our files */
	newf = fopen(mpacktmp,"w");
	mp = fopen(mpackname,"r");

	if (!newf)
	{
		syslog(1,"AM_add_disclaimer: Cannot open %s, %s",mpacktmp,strerror(errno));
		return -1;
	}

 	if (!mp)
   {
		syslog(1,"AM_add_disclaimer: Cannot open %s, %s",mpacktmp,strerror(errno));
		return -1;
	}



	/* Initialise our variables */
	content_type = _none;
	content_encoding = _none;

	if (__verbose) fprintf(stderr,"Adding disclaimer...\n");

	/* Read every line from the original mailpack */
	while ( fgets(line,1024,mp) )
	{

		/* copy and convert into lower-case the input line */
		strcpy(lline,line);
		strcpy(safe,line);
		AM_strlower(lline);

		/* check and see if this is a content-type line */
		if ((pos=strstr(lline, "content-type:")))
		{
			pos += strlen("content-type:");	/* move our pointer along */

			if ((pos=strstr(pos,"text/")))	/* do we have a text type */
			{
				pos += strlen("text/");	/* move pointer along */

				if (strncmp(pos, "plain",5)==0) content_type = _text_plain;
				else
				    if (strncmp(pos, "html",4)==0) content_type = _text_html;
				else
				    if (strncmp(pos, "print",5)==0) content_type = _text_print;
				else
				    content_type = _text_unknown;

			}
			else
			    {
				content_type = _unknown;
			}
			if (__verbose) fprintf(stderr,"Content line found...\n");
		} /* if the line had content-type in it */




		/* Our transfer encoding system indicates if we have a attachment type which is
		* binary-encoded stuff or plain text */
		if ((pos=strstr(lline, "content-transfer-encoding:")))
		{
			pos += strlen("content-transfer-encoding:");
			if (strstr(pos,"base64")) content_encoding = _base64;
			else if (strstr(pos,"quote")) content_encoding = _print;
			else if (strstr(pos,"7bit")) content_encoding = _7bit;
			if (__verbose) fprintf(stderr,"CTE found...\n");
		} /* if the line had content-transfer */



		/* if we have not found our boundary yet */
		if (boundary_found < 1)
		{
			pos = strstr(lline, "boundary=");	/* check for our boundary */

			if (pos)
			{ /* if we found one! */
				if (__verbose) fprintf(stderr,"Possible boundary...");
				newpos = (pos -lline) +line +strlen("boundary=");	/* move the pos pointer to just after the = */
				pos = newpos;
				if (*pos == '\"') pos++;	/* if our 1st char is a quote, move one more */
				newpos = strtok(pos,"\n\r\"");	/* tokenise out the boundary */
				strcpy(boundary, newpos);	/* copy it to our static string array */
				boundary_found++;	/* increment the boundary_found flag */
				if (__verbose) fprintf(stderr,"boundary located...");
			} /* if POS */
		} /* if boundary_found < 1 */
		else
		    {

			/* if we've not yet found a place to put our disclaimer */
			if (boundary_found < 3)
			{
				pos = strstr(line, boundary);
				if (pos)
				{
					boundary_found++;
				}
			}


			/* if we've found the end of the first attachment, ie where we
			* want to put our disclaimer */
			if (boundary_found == 3)
			{
				/* Add in the text only disclaimer */
				/* Add in the text only disclaimer */
				/* Add in the text only disclaimer */
				if (__verbose) fprintf(stdout,"inserting disclaimer...");
				d = fopen(disclaimer,"r");
				AM_insert_disclaimer(d, newf, content_type, content_encoding);
				fclose(d);
				disclaimer_inserted = 1; /* Our disclaimer has been inserted, so set the flag */

				/* Add in the HTML disclaimer (if wanted) */
				/* Add in the HTML disclaimer (if wanted) */
				/* Add in the HTML disclaimer (if wanted) */
				if (__HTMLtoo)
				{

					fprintf(newf,"%s",safe); /* save the original line to the output file */

					/* to ensure we dont try and insert this into a filenamed-attachment
					* we must scan each line for a "filename" until we find our </body>
					* tag */
					while (fgets(line,1024,mp))
					{
						strcpy(lline,line);
						strlower(lline);

						if (strstr(lline,"content-type: text/html"))
						{
							ishtml = 1;
						}
						else if (strstr(lline,"filename="))
						{
							isfile = 1;
						}

						if ((ishtml) && (!isfile))
						{
							if (strstr(lline,"</body>"))
							{
								if (__verbose) fprintf(stdout,"inserting disclaimer (HTML)...");
								d = fopen(disclaimer,"r");
								content_type = _text_html;
								AM_insert_disclaimer(d, newf, content_type, content_encoding);
								fclose(d);
								strcpy(safe,line);
								break;
							}
						}

						fprintf(newf,"%s",line);

					} /* while */
				} /* if HTML too */

			} /* if boundary_found == 3 */

		} /* if-ELSE (boundary) */

		fprintf(newf,"%s",safe); /* save the original line to the output file */

		/* if our disclaimer was inserted, then we will just runn out the last of
		* the MIMEpack lines, so we dont need to process them with strstr()'s as
		* we've already performed the required task */
		if (disclaimer_inserted)
		{
			while (fgets(line,1024,mp)) fprintf(newf,"%s",line);
		}

	} /* WHILE more lines to read in from original */

	/* if we didn't get the disclaimer inserted, then we've encountered a plain-text email
	* which we just need to insert the disclaimer at the end of it */
	
	
	if (!disclaimer_inserted)
	{
		if (__verbose) fprintf(stderr,"No disclaimer added, despite going through entire file...\n");
		d = fopen(disclaimer,"r");
		if (!d)
		{
			fprintf(stderr,"Error Cannot open disclaimer file %s,%s", disclaimer, strerror(errno));
			return -1;
		}
		AM_insert_disclaimer(d, newf, content_type, content_encoding);
		fclose(d);
	}

	if (__verbose) fprintf(stdout,"done.\n");

	/* Close our files */
	fclose(mp);
	fclose(newf);

	/* rename the files */
	rename(mpackname, mpackold);
	rename(mpacktmp, mpackname);


	return result;
}






/*------------------------------------------------------------------------
Procedure:     AM_filename_fix ID:1
Purpose:       Converts a MIME header
Input:         line: string with the filename
removed_prefix: filename prefix to use
removed_count: the number of attachments we've processed / ID
Output:
Errors:
------------------------------------------------------------------------*/
int AM_filename_fix( char *line, char *removed_prefix, int removed_count )
{
	char *pos;
	char lline[1024];

	/* process the current line, as it contains the filename, and replace the filename
	* with a -removed- one */
	pos = strrchr(line,'='); /* locate where the filename starts */
	pos++; /* move one char along */
	*pos = '\0'; /* terminate the string */

	sprintf(lline,"%s\"%s%d\"\n",line,removed_prefix,removed_count); /* create a new string, in lline (scratch pad)*/
	strcpy(line,lline);

	return 0;
}






/*------------------------------------------------------------------------
Procedure:     AM_nullify_attachment ID:1
Purpose:       Removes any attachment within the mailpack which matches
(minimally) with the attachmentname parameter
Input:         mpackname - The mailpack name
attachmentname - The name of attachments which is to be removed
Output:        none
Errors:        If attachment wasn't found, or could not be removed, a non-zero return value.
------------------------------------------------------------------------*/
int AM_nullify_attachment( char *mpackname, char *attachmentname, char *removed_prefix )
{
	FILE *mp, *newf;
	char line[1024]="";
	char safe[1024]="";
	char lline[1024]="";
	char mpacktmp[1024]="";
	char mpackold[1024]="";
	char boundary[1024]="";

	int content_type = _none;
	int content_encoding = _none;

	int result = 0;
	int boundary_found = 0;
	int filename_found = 0;
	int removed_count = 0;
	char *pos, *newpos;

	if (__verbose) fprintf(stdout,"Removing attachments \"%s\"...",attachmentname);

	/* create our temp filename */
	sprintf(mpacktmp,"%s.tmp",mpackname);
	sprintf(mpackold,"%s.old",mpackname);

	/* Initialise our files */
	newf = fopen(mpacktmp,"w");
	mp = fopen(mpackname,"r");

	/* Initialise our variables */
	content_type = _none;
	content_encoding = _none;

	AM_strlower(attachmentname);

	/* Read every line from the original mailpack */
	while ( fgets(line,1024,mp) ){

		/* copy and convert into lower-case the input line */
		strcpy(lline,line);
		strcpy(safe,line);
		AM_strlower(lline);

		/* check and see if we've found the filename... */
		if ( (pos=strstr(lline, "name="))&&(pos=strstr(lline,attachmentname)) )
		{
			filename_found = 1;
			if (__verbose) fprintf(stdout,"located attachment...");
		}

		/* check and see if this is a content-type line */
		if ((pos=strstr(lline, "content-type:")))
		{
			pos += strlen("content-type:");	/* move our pointer along */

			if ((pos=strstr(pos,"text/")))	/* do we have a text type */
			{
				pos += strlen("text/");	/* move pointer along */

				if (strncmp(pos, "plain",5)==0) content_type = _text_plain;
				else
				    if (strncmp(pos, "html",4)==0) content_type = _text_html;
				else
				    if (strncmp(pos, "print",5)==0) content_type = _text_print;
				else content_type = _text_unknown;

			}
			else {
				content_type = _unknown;
			}
		}

		/* Our transfer encoding system indicates if we have a attachment type which is
		* binary-encoded stuff or plain text */
		if ((pos=strstr(lline, "content-transfer-encoding:")))
		{
			pos += strlen("content-transfer-encoding:");
			if (strstr(pos,"base64")) content_encoding = _base64;
			else if (strstr(pos,"quote")) content_encoding = _print;
			else if (strstr(pos,"7bit")) content_encoding = _7bit;
		}


		if (boundary_found < 1)
		{ /* if we have not found our boundary yet */
			pos = strstr(lline, "boundary=");	/* check for our boundary */
			if (pos) { /* if we found one! */

				newpos = (pos -lline) +line +strlen("boundary=");	/* move the pos pointer to just after the = */
				pos = newpos;
				if (*pos == '\"') pos++;	/* if our 1st char is a quote, move one more */
				newpos = strtok(pos,"\n\r\"");	/* tokenise out the boundary */
				strcpy(boundary, newpos);	/* copy it to our static string array */
				boundary_found++;	/* increment the boundary_found flag */
				if (__verbose) fprintf(stdout,"found boundary...");
			} /* if POS */
		} /* if boundary_found < 1 */

		else
		    { /* if boundary_found >= 1 */

			/* if we've found the filename, then we must process until we find the next boundary */
			if (filename_found == 1)
			{
				AM_filename_fix(line, removed_prefix, removed_count);
				fprintf(newf,"%s",line); /* output the new filename */
				if (__verbose) fprintf(stdout,"renamed attachment...");

				/* first, locate our \n\r blank line */

				while ( (fgets(line, 1024,mp)) && (strlen(line) >3) ){
					strcpy(lline,line);
					AM_strlower(lline);

					if (strstr(lline,"name=")){
						AM_filename_fix(line, removed_prefix, removed_count);
					}

					fprintf(newf,"%s",line);
				}


				/* Now, until the boundary is found again, absorb all lines */
				while ( (fgets(line, 1024,mp)) && (!strstr(line, boundary)) );

				/* leave an empty file */
				fprintf(newf,"\n\n");

				/* and, given that we will exit on the boundary line, we can just edit from this loop as we write it later */
				strcpy(safe, line);

				/* we are -done- */
				filename_found = 0;
				removed_count++;
				if (__verbose) fprintf(stdout,"nullified...");
			} /* if filename_found == 1 */

		} /* if-ELSE */

		fprintf(newf,"%s",safe); /* save the original line to the output file */
		filename_found = 0;

	} /* WHILE more lines to read in from original */

	if (__verbose) fprintf(stdout,"done.\n");
	/* Close our files */
	fclose(mp);
	fclose(newf);

	/* Rename files */
	rename(mpackname, mpackold);
	rename(mpacktmp, mpackname);

	return result;
}


/*---------------------------------END-*/



/*----------------------------------------------------------------------
* MIME_insert_X_header
*
* Inserts a specified X-header at the end of other X-headers.
*/
int AM_insert_Xheader( char *fname, char *xheader)
{

	/* Tempfile tmpfile() fix contributed by David DeMaagd - 29/01/2001 */
	char line[1024], lline[1024];
	char tpn[1024];
	int xon = 0;
	int returnvalue = 0;
	int syslogging = 0;
	int stderrlogging = 1;
	FILE *fo;
	FILE *f;

	/* Get temp name */
	sprintf(tpn,"%s.",fname);

	fo = fopen(tpn,"w");
	if (!fo)
	{
		if (syslogging > 0) syslog(1,"Error: Cannot open temporary file for writing (%s) : %s",tpn, strerror(errno));
		if (stderrlogging > 0) fprintf(stderr,"Error: Cannot open temporary file for writing");
		returnvalue = 1;
	}
	f = fopen(fname,"r");
	if (!f)
	{
		if (syslogging > 0) syslog(1,"Error: Cannot open Mailpack file for reading (%s) : %s",fname, strerror(errno));
		if (stderrlogging > 0) fprintf(stderr,"Error: Cannot open Mailpack file for reading, %s",fname);
		returnvalue = 2;
	}

	if ( returnvalue == 0 )
	{
		while (fgets(line,1023,f))
		{
			strncpy(lline,line,1023);
			strlower(lline);

			if ((xon==0)&&(strstr(lline,"content-type:")))
			{
				fprintf(fo,"%s\n",xheader);
				xon=1;
			}

			fprintf(fo,"%s",line);
		}

		/* close files */
		fclose(fo);
		fclose(f);

		/* remove the old file */
		if (remove(fname) == -1) {
			if (stderrlogging > 0) fprintf(stderr,"Error: xInsertHeader - REMOVE, %s",strerror(errno));
		}
		else {
			/* and rename */
			if (rename(tpn,fname) == -1) {
				if (stderrlogging > 0) fprintf(stderr,"Error: xInsertHeader - RENAME, %s",strerror(errno));
			} /*if rename*/
		} /* else */

	} /* if */

	return returnvalue;
}






